function DFBA_DA
% Implementation of the direct approach for the E. coli batch simulation
%  All parameters according to the first case study
%
%  Kai Hoeffner (August, 2012)

%initCobraToolbox
%iJR904=readCBModel('iJR904',2000,'SBML')
%save('iJR904.mat','iJR904')
load iJR904.mat

c=-iJR904.c;
S=iJR904.S;
lb=iJR904.lb;
ub=iJR904.ub;
b=iJR904.b;

% Set initial conditions
Y0 = [0.03 15.5 8 0];
batchtime = [0 25];

options = odeset('Events',@events) ;
[T,Y] = ode15s(@DFBA_DA_ODE,batchtime,Y0,options,1.5,S,b,c,lb,ub);

plot(T,[Y(:,1),Y(:,2),Y(:,3),Y(:,4)])
legend('Biomass','Glucose','Xylose','Ethanol')
xlabel('Time [h]');
ylabel('Concentration [g/L]');
title('E. coli batch simulation');



% --------------------------------------------------------------------------

function dy = DFBA_DA_ODE(t,y,p,S,b,c,lb,ub)
% Dynamic equation for the E. coli batch simulation

% Uptake kinetics parameter
v_gmax = 10.5;
Kg = 0.0027;
Kie = 20;
v_Omax = 15;
KO = 0.024;

v_Zmax = 6;
KZ = 0.0165;
Kig = 0.005;

% Update concentrations
G = y(2);
O = 0;
E = y(4);
Z = y(3);

% Update exchange fluxes
v_g = -v_gmax * G/(Kg + G) * 1/(1 + E/Kie);
v_Z = -v_Zmax * Z/(KZ + Z) * 1/(1 + E/Kie) * 1/(1 + G/Kig);
v_O = -v_Omax * O/(KO + O);

% Glucose flux ID = 344
% Oxygen flux ID  = 392
% Xylose flux ID  = 429
% Ethanol flux ID  = 329

% update lower bounds based on v_g,max, v_O,max and v_Z,max
lb(344) = -v_gmax ;
lb(392) = -v_Omax ;
lb(429) = -v_Zmax ;

A = [S ; zeros(3,size(c,1)) ];
     
A(size(S,1)+1,344) = 1 ;
A(size(S,1)+2,392) = 1 ;
A(size(S,1)+3,429) = 1 ;
b1 = [b ; v_g ; v_O ; v_Z] ;

% Update growth rate and
[v,mu]=cplexlp(c,[],[],A,b1,lb,ub);

if isempty(mu) 
    mu = 0;
    v_e= 0;
else
%   Secondary optimization for ethanol flux    
    A2 = [ c' ; A] ;
    b2 = [ mu ; b1 ] ; 
    c2 = zeros(size(c));
    c2(329) = 1 ;
    [v,v_e]=cplexlp(c2,[],[],A2,b2,lb,ub);
end

% Dynamic equations
dy = zeros(4,1);       
dy(1) = -mu*y(1);      % biomass
dy(2) = 0.18*v_g*y(1); % glucose
dy(3) = 0.15*v_Z*y(1); % xylose
dy(4) = 0.046*v_e*y(1);% ethanol

% --------------------------------------------------------------------------

function [value,isterminal,direction] = events(t,y,p,S,b,c,lb,ub)
% Locate the time when xylose concentration falls below p = 1.5 g/L
% and stop integration.  
value = y(3)-p;  
isterminal = 1;  
direction = -1;